@@ -203,7 +203,7 @@ pub(crate) fn analyze(
         Document::parse(&src).context("Unable to parse the Runefile")?;
     let rune = hotg_rune_syntax::analyse(&parsed, &mut diags);
 
-    let mut writer = StandardStream::stdout(color);
+    let mut writer = StandardStream::stderr(color);
     let config = Config::default();
 
     for diag in &diags {
