@@ -1,19 +1,18 @@
-use std::path::PathBuf;
+use regex::Regex;
+use std::{path::PathBuf, str::FromStr};
 use anyhow::{Context, Error};
 use log;
 use hotg_rune_core::capabilities;
+use once_cell::sync::Lazy;
 use crate::run::{
     Accelerometer, Image, Raw, Sound, accelerometer::Samples,
     image::ImageSource, multi::SourceBackedCapability, new_capability_switcher,
-    runecoral_inference, sound::AudioClip,
+    resources, runecoral_inference, sound::AudioClip,
 };
-use hotg_rune_wasmer_runtime::Runtime;
 use hotg_runicos_base_runtime::{BaseImage, CapabilityFactory, Random};
 
 #[derive(Debug, Clone, PartialEq, structopt::StructOpt)]
 pub struct Run {
-    /// The Rune to run.
-    rune: PathBuf,
     /// The number of times to execute this rune
     #[structopt(short, long, default_value = "1")]
     repeats: usize,
@@ -55,6 +54,25 @@ pub struct Run {
         help = "The librunecoral.so library to use for hardware acceleration"
     )]
     librunecoral: Option<PathBuf>,
+    #[structopt(
+        long,
+        help = "Use the wasm3 WebAssembly engine instead of Wasmer"
+    )]
+    wasm3: bool,
+    #[structopt(
+        long = "file-resource",
+        parse(try_from_str),
+        help = "Load a named resource from a file"
+    )]
+    file_resources: Vec<FileResource>,
+    #[structopt(
+        long = "string-resource",
+        parse(try_from_str),
+        help = "Use the provided string as a resource"
+    )]
+    string_resources: Vec<StringResource>,
+    #[structopt(help = "The Rune to run")]
+    rune: PathBuf,
 }
 
 impl Run {
@@ -65,27 +83,57 @@ impl Run {
             format!("Unable to read \"{}\"", self.rune.display())
         })?;
 
-        let env = self
-            .initialize_image()
-            .context("Unable to initialize the environment")?;
+        let img = self.initialize_image(&rune)?;
 
-        let mut runtime = Runtime::load(&rune, env)
-            .context("Unable to initialize the virtual machine")?;
+        if self.wasm3 {
+            let mut runtime =
+                hotg_rune_wasm3_runtime::Runtime::load(&rune, img)
+                    .context("Unable to initialize the virtual machine")?;
 
-        for i in 0..self.repeats {
-            if i > 0 {
-                log::info!("Call {}", i + 1);
+            for i in 0..self.repeats {
+                if i > 0 {
+                    log::info!("Call {}", i + 1);
+                }
+                runtime.call().context("Call failed")?;
+            }
+        } else {
+            let mut runtime =
+                hotg_rune_wasmer_runtime::Runtime::load(&rune, img)
+                    .context("Unable to initialize the virtual machine")?;
+
+            for i in 0..self.repeats {
+                if i > 0 {
+                    log::info!("Call {}", i + 1);
+                }
+                runtime.call().context("Call failed")?;
             }
-            runtime.call().context("Call failed")?;
         }
 
         Ok(())
     }
 
-    fn initialize_image(&self) -> Result<BaseImage, Error> {
-        // Load the source files for each kind of Capability and create a
-        // CapabilityFactory which will instantiate Capabilities depending on
-        // the "source" index provided as a capability parameter.
+    fn initialize_image(&self, rune: &[u8]) -> Result<BaseImage, Error> {
+        let mut img = BaseImage::with_defaults();
+
+        self.register_capabilities(&mut img)?;
+
+        resources::load_from_custom_sections(&mut img, rune);
+        resources::load_from_files(&mut img, &self.file_resources);
+        resources::load_from_strings(&mut img, &self.string_resources);
+
+        runecoral_inference::override_model_handler(
+            &mut img,
+            self.librunecoral.as_deref(),
+        )
+        .context("Unable to register the librunecoral inference backend")?;
+
+        Ok(img)
+    }
+
+    /// Load the source files for each kind of capability and create a
+    /// [`CapabilityFactory`] which will instantiate capabilities depending on
+    /// the "source" index provided as a capability parameter.
+    fn register_capabilities(&self, img: &mut BaseImage) -> Result<(), Error> {
         let accelerometer = capability_switcher::<Accelerometer, _, _>(
             &self.accelerometer,
             |p| Samples::from_csv_file(p),
@@ -101,19 +149,11 @@ impl Run {
             AudioClip::from_wav_file(p)
         })?;
 
-        let mut img = BaseImage::with_defaults();
-
         img.register_capability(capabilities::ACCEL, accelerometer)
             .register_capability(capabilities::IMAGE, image)
             .register_capability(capabilities::RAW, raw)
             .register_capability(capabilities::SOUND, sound);
 
-        runecoral_inference::override_model_handler(
-            &mut img,
-            self.librunecoral.as_deref(),
-        )
-        .context("Unable to register the librunecoral inference backend")?;
-
         if let Some(seed) = self.random {
             img.register_capability(capabilities::RAND, move || {
                 Ok(Box::new(Random::seeded(seed))
@@ -121,7 +161,7 @@ impl Run {
             });
         }
 
-        Ok(img)
+        Ok(())
     }
 }
 
@@ -130,9 +170,9 @@ impl Run {
 /// capability.
 ///
 /// For example, imagine passing the path for 3 images to the `rune` CLI.
-/// Inside the Rune, we'll instantiate a [`Capability`] object and set the
-/// `"source"` parameter to `1`. This then tells the [`CapabilityFactory`] that
-/// we want to read from the second image.
+/// Inside the Rune, we'll instantiate a [`SourceBackedCapability`] object and
+/// set the `"source"` parameter to `1`. This then tells the
+/// [`CapabilityFactory`] that we want to read from the second image.
 fn capability_switcher<C, T, F>(
     items: &[T],
     mut make_source: F,
@@ -150,3 +190,54 @@ where
 
     Ok(new_capability_switcher::<C, _>(sources))
 }
+
+fn parse_key_value_pair(s: &str) -> Result<(&str, &str), Error> {
+    static PATTERN: Lazy<Regex> =
+        Lazy::new(|| Regex::new(r"([a-zA-Z_][a-zA-Z0-9]*)=(.*)").unwrap());
+
+    let captures = PATTERN
+        .captures(s)
+        .context("Expected a resource in the form \"NAME=value\"")?;
+    let key = captures.get(0).unwrap().as_str();
+    let value = captures.get(1).unwrap().as_str();
+
+    Ok((key, value))
+}
+
+#[derive(Debug, Clone, PartialEq)]
+pub(crate) struct FileResource {
+    pub name: String,
+    pub path: PathBuf,
+}
+
+impl FromStr for FileResource {
+    type Err = Error;
+
+    fn from_str(value: &str) -> Result<Self, Error> {
+        let (name, path) = parse_key_value_pair(value)?;
+
+        Ok(FileResource {
+            name: name.to_string(),
+            path: PathBuf::from(path),
+        })
+    }
+}
+
+#[derive(Debug, Clone, PartialEq)]
+pub(crate) struct StringResource {
+    pub name: String,
+    pub value: String,
+}
+
+impl FromStr for StringResource {
+    type Err = Error;
+
+    fn from_str(value: &str) -> Result<Self, Error> {
+        let (name, value) = parse_key_value_pair(value)?;
+
+        Ok(StringResource {
+            name: name.to_string(),
+            value: value.to_string(),
+        })
+    }
+}
